********************************************************************************************
******** Uncertainty measures from partially rounded probabilistic forecast surveys ********
********************************************************************************************
set type double

**** REMARK: The code below implements counterfactual scenario A from Section 6.1

******************************
******** process data ********
******************************
cd "SET WORKING DIRECTLY TO MAIN-FOLDER"
use "Processed\euro_spf.dta", clear

* drop unemployment forecasts (not needed for comparison since bin width is already the same)
drop if variable=="unemployment"

* replace missing probabilities with zeroes
foreach v of varlist F*{
	replace `v'=0 if `v'==.
}

* combine 44 bins into 22 bins with double bin width
gen FN7_0=0
gen F15_5=0
sum F*
forvalues i=0(1)15{
	cap noisily: gen double from`i'_0=F`i'_0+F`i'_5
}
forvalues i=1(1)7{
	local j=`i'-1
	cap noisily: gen double fromn`i'_0=FN`j'_5+FN`i'_0
}
sum from*
egen pmasstotal=rowtotal(from*)
drop pmasstotal
drop F*

* extend outer bounds (careful: separate rules for X.0 and X.5 needed!)
replace lower_min=lower_min-0.5
replace lower_max=lower_max-0.5 if lower_max==3.5

* drop five-year fixed horizon forecasts (only available in 1999Q1, 2000Q1 and 2001Q1)
drop if target==3

* reshape into long format for each id, so that probabilities are in one column
egen id=group(variable fct_p target fct_s)
reshape long from, i(id) j(binn) string
rename from p

**** split bin names and generate lower and upper bound of bin 
split binn, p("_")
replace binn1=subinstr(binn1,"n","-",.)
gen miss=.
egen lower1=concat(binn1 miss binn2)
drop binn binn1 binn2 miss
destring lower1, replace
gen double lower=lower1
drop lower1
gen double upper=lower+0.9

* extend lower and upper end of interior bins by 0.05 percentage points
replace lower=lower-0.05
replace upper=upper+0.05
replace lower_min=lower_min-0.05
replace lower_max=lower_max-0.05

* group variable for bins
egen binn=group(lower)
sort id binn

* calculate number of bins used
bys id: egen nbins=count(p) if p!=0&p!=.

* calculate total probability mass and drop if forecasters do not report a density forecast or reported probabilities do not sum to one as in Abel et al. (2016)
bys id: egen pmasstotal=sum(p), missing
drop if pmasstotal==.|pmasstotal<=99.1|pmasstotal>=100.9

* replace unused bins of forecasters that report density forecasts with zeroes
*replace p=0 if p==.

* merge realizations
merge m:m variable type target_year using "Processed\euro_actual.dta" 
drop if _merge==2
drop _merge
sort id binn

* drop bins not available 
keep if lower>=lower_min&lower<=lower_max

******** rounding schemes ********

* if left-most (right-most) bin is exterior, set bound one percentage point below (above) the right (left) endpoint of the lowest (highest) bin during the survey history 
* in other words, assume open intervals have twice the length of the other intervals
replace lower=upper-2 if lower==lower_min
replace upper=lower+2 if lower==lower_max

* fix bounds of individual densities at left-most/right-most bin with non-zero probability
drop if p==0

* integer-based: probabilities multiple of tau (note that float has higher precision but trims extreme decimal places)
gen float p2=p
forvalues tau=1(1)10{
	gen mult`tau'=(int(p2/`tau')*`tau')==p2 if p2!=0
	replace mult`tau'=0 if mult`tau'==.
	bys id: egen bin_m`tau'=mode(mult`tau'), min
	replace bin_m`tau'=1-bin_m`tau'
}
drop p2

* count number of decimals of each probability number
tostring p, replace force
split p, p(.)
gen d=0
replace d=strlen(p2)
bys id: egen d_avg=mean(d)

* decimal-based: any, most or all probabilities contain decimals
gen bin_dec=d>0 
bys id: egen nbins_dec=total(bin_dec) 
gen rho=nbins_dec/nbins
gen bin_any=rho>0
gen bin_most=rho>0.5
gen bin_all=rho==1
destring p, replace
drop mult* p1 p2 d bin_dec nbins_dec rho

******** histogram moments ********

* bin midpoint
gen double midpoint=lower+(upper-lower)/2

* histogram mean 
replace p=p/100
bys id: egen point_imp=total(midpoint*p) 

* forecast errors	
gen err=fr-point_imp 
gen sqerr=(fr-point_imp)^2 
gen abserr=abs(fr-point_imp) 

* histogram variance
bys id: egen variance=total((midpoint-point_imp)^2*p)

******** prepare to reshape and rearrange data ********

* suffix to be added to variables in wide format
gen suffix="_g"
replace suffix="_i" if variable=="inflation"

* drop duplicates of each id 
duplicates drop id, force
drop variable p lower* upper midpoint binn pmasstotal* id 

******** reshape and rearrange data ********
egen id=group(fct_p target fct_source)
drop target
sort id suffix 
reshape wide target* cr sr fr point* nbins bin_* d_avg *err variance*, i(id) j(suffix) string
drop id

* new target variable
gen target=.
replace target=1 if type=="fh"&h==4
replace target=2 if type=="fh"&h==8
replace target=3 if type=="fh"&h==20
replace target=4 if type=="fe"&h<=4
replace target=5 if type=="fe"&h>4&h<=8
replace target=6 if type=="fe"&h>8&h<=12
replace target=7 if type=="fe"&h>12

* in the following, focus on this year and next year fixed event forecasts
keep if target==4|target==5

* check for which target years realizations are available
keep if target_year_g>=2000&target_year_g<=2017

* additional variables
foreach x in g i{

	* average point forecast 
	bys t target: egen point_avg_`x'=mean(point_imp_`x')

	* disagreement (variance)
	gen dev_`x'=(point_imp_`x'-point_avg_`x')^2
	bys t target: egen participants_`x'=count(point_imp_`x')
	bys t target: egen sum_`x'=total(dev_`x') if participants_`x'>0
	gen disagreement_`x'=(1/(participants_`x'-1))*sum_`x'
	drop dev_`x' participants_`x' sum_`x'	

	* average uncertainty
	bys t target: egen ubar_`x'=mean(variance_`x')
	bys t target: egen ubar_`x'_rou=mean(variance_`x') if bin_m5_`x'==0
	bys t target: egen ubar_`x'_non=mean(variance_`x') if bin_m5_`x'==1

}

* order data
order t fct_period fct_year fct_quarter type h target fct_source target_period_g target_year_g target_quarter_g fr_g sr_g cr_g point_g nbins_g d_avg_g bin_m1_g-bin_m10_g bin_any_g bin_most_g bin_all_g point_imp_g err_g abserr_g sqerr_g variance_g point_avg_g disagreement_g ubar_g* target_period_i target_year_i target_quarter_i fr_i sr_i cr_i point_i nbins_i d_avg_i bin_m1_i-bin_m10_i bin_any_i bin_most_i bin_all_i point_imp_i err_i abserr_i sqerr_i variance_i point_avg_i disagreement_i ubar_i* 
sort t target fct_source

**** Alternative version of Figure 4: Average ex-ante uncertainty (based on adjusted bins)
twoway (scatter ubar_i target_year_i if h>1&h<8, title("") ytitle("") ytick(0(0.1)0.6) ylabel(0(0.1)0.6, angle(horizontal) labsize(vsmall)) xtitle("") xtick(2000(1)2017) xlabel(2000 "2000" 2004 "2004" 2008 "2008" 2012 "2012" 2016 "2016", labsize(vsmall)) scale(1.5) scheme(s1color) legend(title("`description'", size(medium)) order(0) region(lstyle(none)) position(11) ring(0)) plotr(m(zero)) msymbol(smx) mcolor(gs10)) (scatter ubar_i target_year_i if h==1, mlcolor(black) mfcolor(red)) (scatter ubar_i target_year_i if h==8, mlcolor(black) mfcolor(red) msymbol(triangle))
graph export "Figures\euro_ubar_i_comb.pdf", replace	
twoway (scatter ubar_g target_year_g if h>1&h<8, title("") ytitle("") ytick(0(0.2)0.8) ylabel(0(0.2)0.8, angle(horizontal) labsize(vsmall)) xtitle("") xtick(2000(1)2017) xlabel(2000 "2000" 2004 "2004" 2008 "2008" 2012 "2012" 2016 "2016", labsize(vsmall)) scale(1.5) scheme(s1color) legend(title("`description'", size(medium)) order(0) region(lstyle(none)) position(11) ring(0)) plotr(m(zero)) msymbol(smx) mcolor(gs10)) (scatter ubar_g target_year_g if h==1, mlcolor(black) mfcolor(green)) (scatter ubar_g target_year_g if h==8, mlcolor(black) mfcolor(green) msymbol(triangle))
graph export "Figures\euro_ubar_g_comb.pdf", replace	

******** Section 6.2: Variance misalignment and the width of histogram bins ********
foreach x in i g{
	preserve
	
	**** exclude all forecasts that only use one bin (their uncertainty is zero)
	drop if nbins_`x'==1
	
	**** Variance misalignment in the SPF data (decimal- and integer-based categorization)
	egen mse_`x'=mean(sqerr_`x') if point_imp_`x'!=., by(fct_s h)
	egen var_mean_`x'=mean(variance_`x') if point_imp_`x'!=., by(fct_s h)
	gen m_ind_`x'=mse_`x'/var_mean_`x'	
	
	**** Figures 14 and XX: Adjusted variance misalignment in the SPF data (scenario A, decimal- and integer-based categorization)
	bys h: egen m_`x'_all=mean(m_ind_`x')
	bys h: egen m_`x'_rou_m5=mean(m_ind_`x') if bin_m5_`x'==0
	bys h: egen m_`x'_non_m5=mean(m_ind_`x') if bin_m5_`x'==1
	bys h: egen m_`x'_rou_any=mean(m_ind_`x') if bin_any_`x'==0
	bys h: egen m_`x'_non_any=mean(m_ind_`x') if bin_any_`x'==1
	if "`x'"=="i"{
		local f 0
		local t 4
		local b 1
		local color "red"
	}
	if "`x'"=="g"{
		local f 0
		local t 12
		local b 3
		local color "dkgreen"	
	}
	gen H=-h
	twoway connected m_`x'_all m_`x'_rou_m5 m_`x'_non_m5 H, lcolor(`color' `color' `color') lpattern(solid longdash shortdash) mcolor(`color' `color' `color') msymbol(X triangle square) title("") ytitle("Misalignment ratio", size(small)) ytick(`f'(`b')`t') ylabel(`f'(`b')`t', angle(horizontal) labsize(vsmall)) xtitle("") xtick(-8(1)-1) xlabel(-8 "8" -7 "7" -6 "6" -5 "5" -4 "4" -3 "3" -2 "2" -1 "1", labsize(vsmall)) scale(1.5) scheme(s1color) plotr(m(zero)) yline(1, lcolor(black) lpattern(longdash)) legend(order(1 2 3) label(1 "full sample") label(2 "rounders") label(3 "non-rounders") cols(1) region(lstyle(none)) position(1) ring(0))
	graph export "Figures\euro_ratio_`x'_m5_comb.pdf", replace
	twoway connected m_`x'_all m_`x'_rou_any m_`x'_non_any H, lcolor(`color' `color' `color') lpattern(solid longdash shortdash) mcolor(`color' `color' `color') msymbol(X triangle square) title("") ytitle("Misalignment ratio", size(small)) ytick(`f'(`b')`t') ylabel(`f'(`b')`t', angle(horizontal) labsize(vsmall)) xtitle("") xtick(-8(1)-1) xlabel(-8 "8" -7 "7" -6 "6" -5 "5" -4 "4" -3 "3" -2 "2" -1 "1", labsize(vsmall)) scale(1.5) scheme(s1color) plotr(m(zero)) yline(1, lcolor(black) lpattern(longdash)) legend(order(1 2 3) label(1 "full sample") label(2 "rounders") label(3 "non-rounders") cols(1) region(lstyle(none)) position(1) ring(0))
	graph export "Figures\euro_ratio_`x'_any_comb.pdf", replace
	
	restore
}

window manage close graph
